package com.nokia.financeapi.controller.house;

import com.nokia.financeapi.common.R;
import com.nokia.financeapi.pojo.dto.GetBuildingAreaDiffDto;
import com.nokia.financeapi.pojo.dto.GetHouseAbnormalDataDto;
import com.nokia.financeapi.pojo.vo.GetBuildingAreaDiffVo;
import com.nokia.financeapi.pojo.vo.GetHouseAbnormalDataVo;
import com.nokia.financeapi.service.house.HouseDataCheckService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

@Tag(name = "不动产数据稽查")
@RestController
@RequestMapping("/house-car/house/data-check/api")
public class HouseDataCheckController {
    private final HouseDataCheckService houseDataCheckService;

    public HouseDataCheckController(HouseDataCheckService houseDataCheckService) {
        this.houseDataCheckService = houseDataCheckService;
    }

    @Operation(summary = "查询自用面积比较")
    @PostMapping("/getBuildingAreaDiff")
    public R<GetBuildingAreaDiffVo> getBuildingAreaDiff(@Valid @RequestBody GetBuildingAreaDiffDto dto) {
        return houseDataCheckService.getBuildingAreaDiff(dto);
    }

    @Operation(summary = "自用面积比较导出")
    @GetMapping("/getBuildingAreaDiffExport")
    public void getBuildingAreaDiffExport(@Valid GetBuildingAreaDiffDto dto) {
        houseDataCheckService.getBuildingAreaDiffExport(dto);
    }

    @Operation(summary = "查询异常数据稽核表")
    @PostMapping("/getHouseAbnormalData")
    public R<GetHouseAbnormalDataVo> getHouseAbnormalData(@Valid @RequestBody GetHouseAbnormalDataDto dto) {
        return houseDataCheckService.getHouseAbnormalData(dto);
    }

    @Operation(summary = "异常数据稽核表导出")
    @GetMapping("/getHouseAbnormalDataExport")
    public void getHouseAbnormalDataExport(@Valid GetHouseAbnormalDataDto dto) {
        houseDataCheckService.getHouseAbnormalDataExport(dto);
    }
}
