package com.nokia.financeapi.controller.house;

import com.nokia.financeapi.common.R;
import com.nokia.financeapi.pojo.dto.GetHouseReportDto;
import com.nokia.financeapi.pojo.dto.ImportHouseDataDto;
import com.nokia.financeapi.pojo.vo.GetHouseReportVo;
import com.nokia.financeapi.pojo.vo.GetHouseTemplatesVo;
import com.nokia.financeapi.service.house.HouseReportService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestPart;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.Valid;
import java.util.List;

@Tag(name = "不动产报告")
@RestController
@RequestMapping("/house-car/house/report/api")
public class HouseReportController {
    private final HouseReportService houseReportService;

    public HouseReportController(HouseReportService houseReportService) {
        this.houseReportService = houseReportService;
    }

    @Operation(summary = "获取不动产报告pdf")
    @PostMapping("/getReportPdf")
    public R<GetHouseReportVo> getReportPdf(@Valid @RequestBody GetHouseReportDto dto) {
        return houseReportService.getReportPdf(dto);
    }

    @Operation(summary = "获取不动产报告word")
    @PostMapping("/getReportWord")
    public R<GetHouseReportVo> getReportWord(@Valid @RequestBody GetHouseReportDto dto) {
        return houseReportService.getReportWord(dto);
    }

    @Operation(summary = "获取不动产模板")
    @PostMapping("/getTemplates")
    public R<List<GetHouseTemplatesVo>> getTemplates() {
        return houseReportService.getTemplates();
    }

    @Operation(summary = "导入数据")
    @PostMapping("/importData")
    public R<Object> importData(@Valid ImportHouseDataDto dto, @RequestPart("file") MultipartFile file) {
        return houseReportService.importData(dto, file);
    }
}
