package com.nokia.financeapi.dao.car;

import com.nokia.financeapi.pojo.dto.GetCarCostStatDto;
import com.nokia.financeapi.pojo.dto.GetCarMapStatDto;
import com.nokia.financeapi.pojo.dto.GetCarNoticeDto;
import com.nokia.financeapi.pojo.dto.GetCarTypeStatDto;
import com.nokia.financeapi.pojo.dto.GetDriveStatDto;
import com.nokia.financeapi.pojo.vo.GetCarCostStatVo;
import com.nokia.financeapi.pojo.vo.GetCarMapStatVo;
import com.nokia.financeapi.pojo.vo.GetCarNoticeVo;
import com.nokia.financeapi.pojo.vo.GetCarTypeStatVo;
import com.nokia.financeapi.pojo.vo.GetDriveStatVo;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

import java.util.List;

@Mapper
public interface CarResourceMapDao {
    /**
     * 获取公告
     */
    @Select("""
select * from car.notices
where city = #{dto.city}
and year_month = #{dto.endDate}
""")
    List<GetCarNoticeVo> getNotice(@Param("dto") GetCarNoticeDto dto);

    /**
     * 获取全省或某个地市的车辆类型统计
     */
    @Select("""
<script>
select
    count(che_liang_lei_xing = '皮卡车' or null) as pkc,
    count(che_liang_lei_xing = '微型面包车' or null) as wxmbc,
    count(che_liang_lei_xing = '越野车' or null) as yyc,
    count(che_liang_lei_xing = '轿车' or null) as jc,
    count(che_liang_lei_xing not in ('皮卡车', '微型面包车', '越野车', '轿车') or null) as qtc
from
    car.car_base_data_month
where year_month = #{dto.endDate}
    <if test="dto.city != null and dto.city != ''">
      and first_unit = #{dto.city}
    </if>
</script>
""")
    GetCarTypeStatVo getCarTypeStat(@Param("dto") GetCarTypeStatDto dto);

    /**
     * 获取各个地市的车辆数量和低效数量统计
     */
    @Select("""
with
t101 as (
select
    city,
    count(1) as total
from
    car.car_base_data_month
where
    year_month = #{dto.endDate}
    and city is not null
    and city != ''
group by
    city
),
t102 as (
select
    city,
    count(1) as inefficiency_count
from
    car.car_di_xiao_month
where
    year_month = #{dto.endDate}
    and city is not null
    and city != ''
group by
    city
),
t103 as (
select
t101.*,
t102.inefficiency_count
from t101 left join t102 on t101.city = t102.city
),
t104 as (
select
city as area_name,
total,
coalesce(inefficiency_count, 0) as inefficiency_count
from t103
)
select * from t104 order by area_name
""")
    List<GetCarMapStatVo> getCityCarMapStat(@Param("dto") GetCarMapStatDto dto);

    /**
     * 获取某个地市各区县的车辆数量和低效数量统计
     */
    @Select("""
with
t101 as (
select
    district,
    count(1) as total
from
    car.car_base_data_month
where
    year_month = #{dto.endDate}
    and district is not null
    and district != ''
    and city = #{dto.city}
group by
    district
),
t102 as (
select
    district,
    count(1) as inefficiency_count
from
    car.car_di_xiao_month
where
    year_month = #{dto.endDate}
    and district is not null
    and district != ''
    and city = #{dto.city}
group by
    district
),
t103 as (
select
t101.*,
t102.inefficiency_count
from t101 left join t102 on t101.district = t102.district
),
t104 as (
select
district as area_name,
total,
coalesce(inefficiency_count, 0) as inefficiency_count
from t103
)
select * from t104 order by area_name
""")
    List<GetCarMapStatVo> getDistrictCarMapStat(@Param("dto") GetCarMapStatDto dto);

    /**
     * 获取各个地市的行驶统计
     */
    @Select("""
with
t101 as (
select
    first_unit,
    avg(zong_li_cheng) as mileage,
    avg(chu_qin_lv) as attendanceRate
from
    car.car_li_cheng_month
where
    year_month >= #{dto.startDate}
    and year_month <= #{dto.endDate}
    and first_unit is not null
    and first_unit != ''
    and first_unit != '省公司本部'
    and position('建设部' in che_liang_suo_shu_dan_wei) = 0
group by
    first_unit
),
t102 as (
select
    t101.*,
    car.car_second_unit_sort.sort
from
    t101
left join car.car_second_unit_sort on
    t101.first_unit = car.car_second_unit_sort.second_unit
),
t103 as (
select
    first_unit as area_name,
    round(mileage,
    2) as mileage,
    round(attendanceRate * 100,
    2) as attendanceRate,
    sort
from
    t102
order by
    sort desc
)
select
    *
from
    t103
""")
    List<GetDriveStatVo> getCityDriveStat(@Param("dto") GetDriveStatDto dto);

    /**
     * 获取某个地市各个区县的行驶统计
     */
    @Select("""
with
t101 as (
select
    second_unit,
    avg(zong_li_cheng) as mileage,
    avg(chu_qin_lv) as attendanceRate
from
    car.car_li_cheng_month
where
    year_month >= #{dto.startDate}
    and year_month <= #{dto.endDate}
    and second_unit is not null
    and second_unit != ''
    and first_unit = #{dto.city}
group by
    second_unit
),
t102 as (
select
    second_unit as area_name,
    round(mileage,
    2) as mileage,
    round(attendanceRate * 100,
    2) as attendanceRate
from
    t101
order by
    second_unit
)
select
    *
from
    t102
""")
    List<GetDriveStatVo> getDistrictDriveStat(@Param("dto") GetDriveStatDto dto);

    /**
     * 获取各个地市的费用统计
     */
    @Select("""
with
t101 as (
select
    first_unit,
    sum(zong_fei_yong) as cost_sum,
    avg(zong_fei_yong) as cost_avg
from
    car.car_fei_yong_month
where
    year_month >= #{dto.startDate}
    and year_month <= #{dto.endDate}
    and first_unit is not null
    and first_unit != ''
    and position('建设部' in che_liang_suo_shu_dan_wei) = 0
    and rui_xing = 1
group by
    first_unit
),
t102 as (
select
    t101.*,
    car.car_second_unit_sort.sort
from
    t101
left join car.car_second_unit_sort on
    t101.first_unit = car.car_second_unit_sort.second_unit
),
t103 as (
select
    first_unit as area_name,
    round(cost_sum,
    2) as cost_sum,
    round(cost_avg,
    2) as cost_avg,
    sort
from
    t102
order by
    sort desc
)
select
    *
from
    t103
""")
    List<GetCarCostStatVo> getCityCostStat(@Param("dto") GetCarCostStatDto dto);

    /**
     * 获取某个地市各个区县的费用统计
     */
    @Select("""
with
t101 as (
select
    second_unit,
    sum(zong_fei_yong) as cost_sum,
    avg(zong_fei_yong) as cost_avg
from
    car.car_fei_yong_month
where
    year_month >= #{dto.startDate}
    and year_month <= #{dto.endDate}
    and second_unit is not null
    and second_unit != ''
    and rui_xing = 1
    and first_unit = #{dto.city}
group by
    second_unit
),
t102 as (
select
    second_unit as area_name,
    round(cost_sum,
    2) as cost_sum,
    round(cost_avg,
    2) as cost_avg
from
    t101
order by
    second_unit
)
select
    *
from
    t102
""")
    List<GetCarCostStatVo> getDistrictCostStat(@Param("dto") GetCarCostStatDto dto);
}
