package com.nokia.financeapi.dao.house;

import com.nokia.financeapi.pojo.dto.GetBuildingAreaDiffDto;
import com.nokia.financeapi.pojo.dto.GetHouseAbnormalDataDto;
import com.nokia.financeapi.pojo.vo.GetBuildingAreaDiffVo;
import com.nokia.financeapi.pojo.vo.GetHouseAbnormalDataVo;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

import java.util.List;

@Mapper
public interface HouseDataCheckDao {

    /**
     * 建筑面积变化
     */
    @Select("""
<script>
with
t101 as (
select
    area_no,
    city_no,
    sum(building_area) as building_area_sum_last,
    sum(building_area_self_use) as building_area_self_use_sum_last,
    sum(building_area_idle) as building_area_idle_sum_last
from
    house.building_month
where
    year_month = 202312
group by
    area_no,
    city_no
),
t102 as (
select
    area_no,
    city_no,
    sum(building_area) as building_area_sum_now,
    sum(building_area_self_use) as building_area_self_use_sum_now,
    sum(building_area_idle) as building_area_idle_sum_now
from
    house.building_month
<choose>
  <when test="dto.endDate != null">
    where year_month = #{dto.endDate}
  </when>
  <otherwise>
    where year_month = (select max(year_month) from house.building_month)
  </otherwise>
</choose>
<if test="dto.areaNo != null and dto.areaNo != ''">
  and area_no = #{dto.areaNo}
</if>
<if test="dto.cityNo != null and dto.cityNo != ''">
  and city_no = #{dto.cityNo}
</if>
group by
    area_no,
    city_no
),
t103 as (
select
    b.id as area_no,
    b."name" as area_name,
    a.id as city_no,
    a."name" as city_name,
    a."name" as unit_name,
    coalesce(round(t101.building_area_sum_last,
    2),
    0) as building_area_sum_last,
    coalesce(round(t101.building_area_self_use_sum_last,
    2),
    0) as building_area_self_use_sum_last,
    coalesce(round(t101.building_area_idle_sum_last,
    2),
    0) as building_area_idle_sum_last,
    coalesce(round(t102.building_area_sum_now,
    2),
    0) as building_area_sum_now,
    coalesce(round(t102.building_area_self_use_sum_now,
    2),
    0) as building_area_self_use_sum_now,
    coalesce(round(t102.building_area_idle_sum_now,
    2),
    0) as building_area_idle_sum_now,
    case
        when t101.building_area_sum_last = 0 then null
        else round((coalesce(t102.building_area_sum_now,
        0) - coalesce(t101.building_area_sum_last,
        0)) / t101.building_area_sum_last * 100,
        2)
    end as building_area_diff,
    case
        when t101.building_area_self_use_sum_last = 0 then null
        else round((coalesce(t102.building_area_self_use_sum_now,
        0) - coalesce(t101.building_area_self_use_sum_last,
        0)) / t101.building_area_self_use_sum_last * 100,
        2)
    end as building_area_self_use_diff,
    case
        when t101.building_area_idle_sum_last = 0 then null
        else round((coalesce(t102.building_area_idle_sum_now,
        0) - coalesce(t101.building_area_idle_sum_last,
        0)) / t101.building_area_idle_sum_last * 100,
        2)
    end as building_area_idle_diff
from
    common.organization a
left join common.organization b on
    a.parent_id = b.id
left join t101 on
    a.id = t101.city_no
left join t102 on
    a.id = t102.city_no
where
    a.grade = 2
    and a.unhide = 1
<if test="dto.areaNo != null and dto.areaNo != ''">
  and a.parent_id = #{dto.areaNo}
</if>
<if test="dto.cityNo != null and dto.cityNo != ''">
  and a.id = #{dto.cityNo}
</if>
order by
    b.order_num,
    a.order_num
)
select
    *
from
    t103
</script>
""")
    List<GetBuildingAreaDiffVo> buildingAreaDiff(@Param("dto") GetBuildingAreaDiffDto dto);

    /**
     * 查询不动产异常数据稽核
     */
    @Select("""
<script>
with
t101 as (
select
    area_no,
    city_no,
    sum(wei_guan_lian_ju_zhi) as wei_guan_lian_ju_zhi_sum,
    sum(xian_zhi_jian_zhu_mian_ji) as xian_zhi_jian_zhu_mian_ji_sum
from
    house.abnormal_data
<choose>
  <when test="dto.endDate != null">
    where year_month = #{dto.endDate}
  </when>
  <otherwise>
    where year_month = (select max(year_month) from house.abnormal_data)
  </otherwise>
</choose>
<if test="dto.areaNo != null and dto.areaNo != ''">
  and area_no = #{dto.areaNo}
</if>
<if test="dto.cityNo != null and dto.cityNo != ''">
  and city_no = #{dto.cityNo}
</if>
group by
    area_no,
    city_no
),
t102 as (
select
    b.id as area_no,
    b."name" as area_name,
    a.id as city_no,
    a."name" as city_name,
    a."name" as unit_name,
    coalesce(c.wei_guan_lian_ju_zhi_sum, 0) as wei_guan_lian_ju_zhi_sum,
    coalesce(c.xian_zhi_jian_zhu_mian_ji_sum, 0) as xian_zhi_jian_zhu_mian_ji_sum
from
    common.organization a
left join common.organization b on
    a.parent_id = b.id
left join t101 c on
    a.id = c.city_no
where
    a.grade = 2
    and a.unhide = 1
<if test="dto.areaNo != null and dto.areaNo != ''">
  and a.parent_id = #{dto.areaNo}
</if>
<if test="dto.cityNo != null and dto.cityNo != ''">
  and a.id = #{dto.cityNo}
</if>
order by
    b.order_num, a.order_num
)
select * from t102
where exists (select 1 from t101)
</script>
""")
    List<GetHouseAbnormalDataVo> getAbnormalData(@Param("dto") GetHouseAbnormalDataDto dto);
}
