package com.nokia.financeapi.service.car;

import com.nokia.financeapi.common.R;
import com.nokia.financeapi.dao.car.CarResourceMapDao;
import com.nokia.financeapi.pojo.dto.GetCarCostStatDto;
import com.nokia.financeapi.pojo.dto.GetCarMapStatDto;
import com.nokia.financeapi.pojo.dto.GetCarNoticeDto;
import com.nokia.financeapi.pojo.dto.GetCarTypeStatDto;
import com.nokia.financeapi.pojo.dto.GetDriveStatDto;
import com.nokia.financeapi.pojo.vo.GetCarCostStatVo;
import com.nokia.financeapi.pojo.vo.GetCarMapStatVo;
import com.nokia.financeapi.pojo.vo.GetCarNoticeVo;
import com.nokia.financeapi.pojo.vo.GetCarTypeStatVo;
import com.nokia.financeapi.pojo.vo.GetDriveStatVo;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.List;

@Service
public class CarResourceMapService {
    private final CarResourceMapDao carResourceMapDao;

    public CarResourceMapService(CarResourceMapDao carResourceMapDao) {
        this.carResourceMapDao = carResourceMapDao;
    }

    public R<List<GetCarNoticeVo>> getNotice(GetCarNoticeDto dto) {
        if (!StringUtils.hasText(dto.getCity())) {
            dto.setCity("全省");
        }
        List<GetCarNoticeVo> vo = carResourceMapDao.getNotice(dto);
        return R.ok(vo);
    }

    public R<GetCarTypeStatVo> getCarTypeStat(GetCarTypeStatDto dto) {
        GetCarTypeStatVo vo = carResourceMapDao.getCarTypeStat(dto);
        return R.ok(vo);
    }

    public R<List<GetCarMapStatVo>> getCarMapStat(GetCarMapStatDto dto) {
        if (StringUtils.hasText(dto.getCity())) {
            List<GetCarMapStatVo> vo = carResourceMapDao.getDistrictCarMapStat(dto);
            return R.ok(vo);
        }
        List<GetCarMapStatVo> vo = carResourceMapDao.getCityCarMapStat(dto);
        return R.ok(vo);
    }

    public R<List<GetDriveStatVo>> getDriveStat(GetDriveStatDto dto) {
        Integer endDate = dto.getEndDate();
        LocalDate endLocalDate = LocalDate.parse(endDate + "01", DateTimeFormatter.ofPattern("yyyyMMdd"));
        LocalDate startLocalDate = endLocalDate.withDayOfYear(1);
        Integer startDate = Integer.valueOf(startLocalDate.format(DateTimeFormatter.ofPattern("yyyyMM")));
        dto.setStartDate(startDate);
        if (StringUtils.hasText(dto.getCity())) {
            List<GetDriveStatVo> vo = carResourceMapDao.getDistrictDriveStat(dto);
            return R.ok(vo);
        }
        List<GetDriveStatVo> vo = carResourceMapDao.getCityDriveStat(dto);
        return R.ok(vo);
    }

    public R<List<GetCarCostStatVo>> getCarCostStat(GetCarCostStatDto dto) {
        Integer endDate = dto.getEndDate();
        LocalDate endLocalDate = LocalDate.parse(endDate + "01", DateTimeFormatter.ofPattern("yyyyMMdd"));
        LocalDate startLocalDate = endLocalDate.withDayOfYear(1);
        Integer startDate = Integer.valueOf(startLocalDate.format(DateTimeFormatter.ofPattern("yyyyMM")));
        dto.setStartDate(startDate);
        if (StringUtils.hasText(dto.getCity())) {
            List<GetCarCostStatVo> vo = carResourceMapDao.getDistrictCostStat(dto);
            return R.ok(vo);
        }
        List<GetCarCostStatVo> vo = carResourceMapDao.getCityCostStat(dto);
        return R.ok(vo);
    }
}
