package com.nokia.financeapi.service.common;

import com.nokia.financeapi.dao.common.OrganizationDao;
import com.nokia.financeapi.pojo.po.common.OrganizationPo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
public class OrganizationService {
    private final OrganizationDao organizationDao;

    public OrganizationService(OrganizationDao organizationDao) {
        this.organizationDao = organizationDao;
    }

    /**
     * 查询二级组织机构
     */
    public List<OrganizationPo> getSecondOrgs() {
        return organizationDao.getSecondOrgs();
    }

    /**
     * 查询三级组织机构
     */
    public List<OrganizationPo> getThirdOrgs() {
        return organizationDao.getThirdOrgs();
    }

    /**
     * 获取组织机构字典，key：id，value：组织机构
     * @param secondOrgs 二级组织机构列表
     * @param thirdOrgs 三级组织机构列表
     * @return {@link Map}<{@link String}, {@link OrganizationPo}>
     */
    public Map<String, OrganizationPo> getOrgMap(List<OrganizationPo> secondOrgs,
                                                 List<OrganizationPo> thirdOrgs) {
        Map<String, OrganizationPo> map = new HashMap<>();
        secondOrgs.forEach(t -> map.put(t.getId(), t));
        thirdOrgs.forEach(t -> map.put(t.getId(), t));
        return map;
    }

    /**
     * 获取三级组织机构列表字典，key：二级组织机构id，value：三级组织机构列表
     * @param secondOrgs 二级组织机构列表
     * @param thirdOrgs 三级组织机构列表
     */
    public Map<String, List<OrganizationPo>> getThirdOrganizationListMap(List<OrganizationPo> secondOrgs,
                                                                         List<OrganizationPo> thirdOrgs) {
        Map<String, List<OrganizationPo>> map = new HashMap<>();
        for (OrganizationPo secondOrg : secondOrgs) {
            map.put(secondOrg.getId(), new ArrayList<>());
            for (OrganizationPo thirdOrg : thirdOrgs) {
                if (secondOrg.getId().equals(thirdOrg.getParentId())) {
                    map.get(secondOrg.getId()).add(thirdOrg);
                }
            }
        }
        return map;
    }
}
