package com.nokia.financeapi.service.common.file;

import cn.hutool.core.util.IdUtil;

import java.io.InputStream;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Map;
import java.util.concurrent.TimeUnit;

public interface FileService {
    /**
     * 生成文件名
     *
     * @return {@link String}
     */
    default String generateFilename() {
        return generateFilename("");
    }

    /**
     * 生成文件名
     *
     * @param suffix 后缀
     * @return {@link String}
     */
    default String generateFilename(String suffix) {
        LocalDateTime now = LocalDateTime.now();
        return now.getYear() + "/" + now.getMonthValue() + "/" + now.getDayOfMonth() + "/"
                + IdUtil.fastSimpleUUID() + "-" + now.format(DateTimeFormatter.ofPattern("yyyyMMddHHmmss")) + suffix;
    }

    /**
     * 获取下载地址
     *
     * @param object 对象名称
     * @return {@link String}
     */
    String getDownloadUrl(String object);

    /**
     * 获取下载地址
     *
     * @param object   对象名称
     * @param duration 过期时间
     * @param unit     单位
     * @return {@link String}
     */
    String getDownloadUrl(String object, int duration, TimeUnit unit);

    /**
     * 从流中上传数据到对象
     *
     * @param object     对象名称
     * @param stream     流
     * @param objectSize 对象大小
     * @param tags       标签
     * @return {@link Map}<{@link String}, {@link Object}>
     */
    Map<String, Object> putObject(String object, InputStream stream, long objectSize, Map<String, String> tags);
    /**
     * 从流中上传数据到对象
     *
     * @param object     对象名称
     * @param stream     流
     * @param objectSize 对象大小
     * @return {@link Map}<{@link String}, {@link Object}>
     */
    default Map<String, Object> putObject(String object, InputStream stream, long objectSize) {
        return putObject(object, stream, objectSize, null);
    }

    /**
     * 获取存储桶名称
     *
     * @return {@link String}
     */
    String getBucket();
}
