package com.nokia.financeapi.service.house;

import com.nokia.financeapi.common.R;
import com.nokia.financeapi.config.HousePages;
import com.nokia.financeapi.dao.common.AreaDao;
import com.nokia.financeapi.dao.house.HouseResourceMapMapper;
import com.nokia.financeapi.pojo.dto.GetBuildingAreaStatDto;
import com.nokia.financeapi.pojo.dto.GetBuildingMapDto;
import com.nokia.financeapi.pojo.dto.GetBuildingOptionsDto;
import com.nokia.financeapi.pojo.dto.GetBuildingRepairStatDto;
import com.nokia.financeapi.pojo.dto.GetHouseNoticeDto;
import com.nokia.financeapi.pojo.dto.GetLandBuildingStatDto;
import com.nokia.financeapi.pojo.dto.GetRentOutStatDto;
import com.nokia.financeapi.pojo.po.common.AreaPo;
import com.nokia.financeapi.pojo.vo.GetBuildingAreaStatVo;
import com.nokia.financeapi.pojo.vo.GetBuildingMapVo;
import com.nokia.financeapi.pojo.vo.GetBuildingOptionsVo;
import com.nokia.financeapi.pojo.vo.GetBuildingRepairStatVo;
import com.nokia.financeapi.pojo.vo.GetHouseNoticeVo;
import com.nokia.financeapi.pojo.vo.GetLandBuildingStatVo;
import com.nokia.financeapi.pojo.vo.GetRentOutStatVo;
import com.nokia.financeapi.pojo.vo.TreeAreaVo;
import com.nokia.financeapi.service.common.file.FileService;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.validation.Valid;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class HouseResourceMapService {
    static final String DEFAULT_BUILDING_IMG = "/oss/public/default-building-img.png";
    private final HouseResourceMapMapper houseResourceMapMapper;
    private final AreaDao areaDao;
    private final FileService fileService;
    private final HousePages housePages;

    public HouseResourceMapService(HouseResourceMapMapper houseResourceMapMapper, AreaDao areaDao, FileService fileService,
                                   HousePages housePages) {
        this.houseResourceMapMapper = houseResourceMapMapper;
        this.areaDao = areaDao;
        this.fileService = fileService;
        this.housePages = housePages;
    }

    public R<GetBuildingAreaStatVo> getBuildingAreaStat(GetBuildingAreaStatDto dto) {
        GetBuildingAreaStatVo vo = houseResourceMapMapper.getBuildingAreaStat(dto);
        return R.ok(vo);
    }

    public R<List<GetLandBuildingStatVo>> getLandBuildingStat(GetLandBuildingStatDto dto) {
        if (StringUtils.hasText(dto.getCity())) {
            List<GetLandBuildingStatVo> vo = houseResourceMapMapper.getDistrictLandBuildingStat(dto);
            return R.ok(vo);
        }
        List<GetLandBuildingStatVo> vo = houseResourceMapMapper.getCityLandBuildingStat(dto);
        return R.ok(vo);
    }

    public R<TreeAreaVo> treeArea() {
        // 查询所有地区
        List<AreaPo> areas = areaDao.getAll();
        Map<String, List<AreaPo>> parentMap = getAreaParentMap(areas);
        // 地市列表
        List<AreaPo> cityList = parentMap.get("130000");
        TreeAreaVo vo = new TreeAreaVo();
        vo.setLabel("河北省");
        vo.setValue("130000");
        List<TreeAreaVo> cityVoList = new ArrayList<>();
        vo.setChildren(cityVoList);
        for (AreaPo tt : cityList) {
            List<TreeAreaVo> districtVoList = new ArrayList<>();
            TreeAreaVo cityVo = new TreeAreaVo();
            cityVo.setLabel(tt.getAreaName());
            cityVo.setValue(tt.getAreaId());
            cityVo.setChildren(districtVoList);
            cityVoList.add(cityVo);
            // 区县列表
            List<AreaPo> districtList = parentMap.get(tt.getAreaId());
            for (AreaPo ttt : districtList) {
                TreeAreaVo districtVo = new TreeAreaVo();
                districtVo.setLabel(ttt.getAreaName());
                districtVo.setValue(ttt.getAreaId());
                districtVoList.add(districtVo);
            }
        }
        TreeAreaVo sbb = new TreeAreaVo();
        sbb.setLabel("省本部");
        sbb.setValue("-12");
        cityVoList.add(sbb);
        return R.ok(vo);
    }

    /**
     * 得到地区父地图
     *
     * @param areas 区域
     */
    private Map<String, List<AreaPo>> getAreaParentMap(List<AreaPo> areas) {
        Map<String, List<AreaPo>> parentMap = new HashMap<>();
        for (AreaPo t : areas) {
            parentMap.putIfAbsent(t.getParentId(), new ArrayList<>());
            parentMap.get(t.getParentId()).add(t);
        }
        return parentMap;
    }

    public R<List<GetBuildingMapVo>> getBuildingMap(GetBuildingMapDto dto, String token) {
        Integer endDate = dto.getEndDate();
        LocalDate endLocalDate = LocalDate.parse(endDate + "01", DateTimeFormatter.ofPattern("yyyyMMdd"));
        LocalDate startLocalDate = endLocalDate.withDayOfYear(1);
        Integer startDate = Integer.valueOf(startLocalDate.format(DateTimeFormatter.ofPattern("yyyyMM")));
        dto.setStartDate(startDate);
        // 坐标为空根据区县获取建筑
        if (dto.getLeftLng() == null || dto.getRightLng() == null || dto.getLeftLat() == null || dto.getRightLat() == null) {
            if ("省本部".equals(dto.getCity())) {
                List<GetBuildingMapVo> vo = houseResourceMapMapper.getSbbBuildings(dto);
                for (GetBuildingMapVo t : vo) {
                    String buildingImg = getBuildingImg(t.getBuildingImg());
                    t.setBuildingImg(buildingImg);
                    String siteUrl = getSiteUrl(token, t.getSiteNum(), endDate);
                    t.setSiteUrl(siteUrl);
                }
                return R.ok(vo);
            }
            List<GetBuildingMapVo> vo = houseResourceMapMapper.getBuildingByDistrict(dto);
            for (GetBuildingMapVo t : vo) {
                String buildingImg = getBuildingImg(t.getBuildingImg());
                t.setBuildingImg(buildingImg);
                String siteUrl = getSiteUrl(token, t.getSiteNum(), endDate);
                t.setSiteUrl(siteUrl);
            }
            return R.ok(vo);
        }
        // 查询坐标范围内的建筑
        List<GetBuildingMapVo> vo = houseResourceMapMapper.getBuildingByCoordinate(dto);
        for (GetBuildingMapVo t : vo) {
            String buildingImg = getBuildingImg(t.getBuildingImg());
            t.setBuildingImg(buildingImg);
            String siteUrl = getSiteUrl(token, t.getSiteNum(), endDate);
            t.setSiteUrl(siteUrl);
        }
        return R.ok(vo);
    }

    public R<List<GetBuildingRepairStatVo>> getBuildingRepairStat(@Valid GetBuildingRepairStatDto dto) {
        Integer endDate = dto.getEndDate();
        LocalDate endLocalDate = LocalDate.parse(endDate + "01", DateTimeFormatter.ofPattern("yyyyMMdd"));
        LocalDate startLocalDate = endLocalDate.withDayOfYear(1);
        Integer startDate = Integer.valueOf(startLocalDate.format(DateTimeFormatter.ofPattern("yyyyMM")));
        dto.setStartDate(startDate);
        if (StringUtils.hasText(dto.getCity())) {
            List<GetBuildingRepairStatVo> vo = houseResourceMapMapper.getThirdUnitRepairStat(dto);
            return R.ok(vo);
        }
        List<GetBuildingRepairStatVo> vo = houseResourceMapMapper.getSecondUnitRepairStat(dto);
        return R.ok(vo);
    }

    public R<List<GetHouseNoticeVo>> getNotice(GetHouseNoticeDto dto) {
        if (!StringUtils.hasText(dto.getCity())) {
            dto.setCity("全省");
        }
        List<GetHouseNoticeVo> vo = houseResourceMapMapper.getNotice(dto);
        return R.ok(vo);
    }

    public R<List<GetRentOutStatVo>> getRentOutStat(GetRentOutStatDto dto) {
        if (StringUtils.hasText(dto.getCity())) {
            List<GetRentOutStatVo> vo = houseResourceMapMapper.getThirdUnitRentOutIncomeStat(dto);
            return R.ok(vo);
        }
        List<GetRentOutStatVo> vo = houseResourceMapMapper.getSecondUnitRentOutIncomeStat(dto);
        return R.ok(vo);
    }

    public R<List<GetBuildingOptionsVo>> getBuildingOptions(GetBuildingOptionsDto dto, String token) {
        Integer endDate = dto.getEndDate();
        LocalDate endLocalDate = LocalDate.parse(endDate + "01", DateTimeFormatter.ofPattern("yyyyMMdd"));
        LocalDate startLocalDate = endLocalDate.withDayOfYear(1);
        Integer startDate = Integer.valueOf(startLocalDate.format(DateTimeFormatter.ofPattern("yyyyMM")));
        dto.setStartDate(startDate);
        dto.setStartDate(startDate);
        if ("省本部".equals(dto.getCity())) {
            List<GetBuildingOptionsVo> vo = houseResourceMapMapper.getSbbBuildingOptions(dto);
            for (GetBuildingOptionsVo t : vo) {
                String buildingImg = getBuildingImg(t.getBuildingImg());
                t.setBuildingImg(buildingImg);
                String siteUrl = getSiteUrl(token, t.getSiteNum(), endDate);
                t.setSiteUrl(siteUrl);
            }
            return R.ok(vo);
        }
        List<GetBuildingOptionsVo> vo = houseResourceMapMapper.getBuildingOptions(dto);
        for (GetBuildingOptionsVo t : vo) {
            String buildingImg = getBuildingImg(t.getBuildingImg());
            t.setBuildingImg(buildingImg);
            String siteUrl = getSiteUrl(token, t.getSiteNum(), endDate);
            t.setSiteUrl(siteUrl);
        }
        return R.ok(vo);
    }

    private String getSiteUrl(String token, String siteNum, Integer endDate) {
        String endDateString = endDate == null ? "" : endDate.toString();
        return housePages.getSiteUrl() + "?data=" + token + "&statistical_month=" + endDateString
                + "&building_address_number=" + siteNum;
    }

    private String getBuildingImg(String buildingImg) {
        return fileService.getBucket() + (StringUtils.hasText(buildingImg) ? buildingImg : DEFAULT_BUILDING_IMG);
    }
}
